""" buffer_unittest.py: unit tests for buffers.py

Tests classes and functions in buffers.py

Should be able run from python2 and python3!

For help see :py:mod:`unittest`

.. codeauthor:: Christian Herdtweck, christian.herdtweck@intra2net.com
"""

import unittest
from math import log, floor

import buffers


class CircBufferTester(unittest.TestCase):
    """ test class CircularBuffer """

    def test_illegal_size(self):
        for size in (-10, -1, 0):
            self.assertRaises(ValueError, buffers.CircularBuffer, size)

    def test_basic(self):
        """ tests basic functionality """

        for size in (1, 2, 4, 8, 16, 32):
            buf = buffers.CircularBuffer(size)
            self.assertTrue(all(item == None for item in buf.get_all()))

            for elem in range(size*2):
                buf.add(elem)
                elems = buf.get_all()
                start_elem = max(0, elem-size+1)
                self.assertEqual(elems[:elem+1], range(start_elem, elem+1))
                self.assertTrue(all(elem == None for elem in elems[elem+1:]))


def log2(x):
    """ Log2 does not exist before python3.3 """
    return log(x, 2)


class LogarithmicBufferTest(unittest.TestCase):
    """ test class LogarithmicBuffer """

    def test_illegal_size(self):
        for size in (-10, -1):
            self.assertRaises(ValueError, buffers.LogarithmicBuffer, size)

    def test_basic(self):
        for size_new in (0, 1, 2, 10):
            buf = buffers.LogarithmicBuffer(size_new)

            for elem in range(1000):
                buf.add(elem)

                # test size
                n_added = elem+1
                n_in_old = n_added - size_new
                if n_in_old <= 0:
                    self.assertEqual(buf.get_size(), size_new)
                elif n_in_old == 1:
                    self.assertEqual(buf.get_size(), size_new + 1)
                else:
                    self.assertEqual(buf.get_size(),
                                     size_new + floor(log2(n_in_old-1))+1)

                # test new contents really is the newest
                start = max(0, elem-size_new)
                expect_new = range(start, elem+1)
                self.assertEqual(buf.get_all()[-len(expect_new):], expect_new)


if __name__ == '__main__':
    unittest.main()
